
function onDataReceived(data) {
    var timeline = d3.select("#timeline");
    var w = Number(timeline.style("width").slice(0,-2));
    // the SVG fills the width of the whole div, but it will extend
    // as far vertically as necessary (depends upon the data)
    var chart = timeline.append("svg:svg")
         .attr("id", "outer_chart")
         .attr("width", w)
         .attr("pointer-events", "all")
        .append("svg:g")
         .call(d3.behavior.zoom().on("zoom", pan_and_zoom))
    ;
    // this "backboard" rect lets us catch mouse events anywhere in the
    // chart, even between the bars. Without it, we only see events on solid
    // objects like bars and text, but not in the gaps between.
    chart.append("svg:rect")
        .attr("id", "outer_rect")
        .attr("width", w).attr("height", 200).attr("fill", "none");

    // but the stuff we put inside it should have some room
    w = w-50;

    // at this point we assume the data is fixed, but the zoom/pan is not

    // create the static things (those which don't exist or not exist based
    // upon the timeline data). Their locations will be adjusted later,
    // during redraw, when we know the x+y coordinates
    chart.append("svg:text")
        .attr("class", "dyhb-label")
        //.attr("x", "20px").attr("y", y)
        .attr("text-anchor", "start") // anchor at top-left
        .attr("dy", ".71em")
        .attr("fill", "black")
        .text("DYHB requests");

    chart.append("svg:text")
        .attr("class", "read-label")
        //.attr("x", "20px").attr("y", y)
        .attr("text-anchor", "start") // anchor at top-left
        .attr("dy", ".71em")
        .attr("fill", "black")
        .text("read() requests");
    chart.append("svg:text")
        .attr("class", "segment-label")
        //.attr("x", "20px").attr("y", y)
        .attr("text-anchor", "start") // anchor at top-left
        .attr("dy", ".71em")
        .attr("fill", "black")
        .text("segment() requests");
    chart.append("svg:text")
        .attr("class", "block-label")
        //.attr("x", "20px").attr("y", y)
        .attr("text-anchor", "start") // anchor at top-left
        .attr("dy", ".71em")
        .attr("fill", "black")
        .text("block() requests");
    chart.append("svg:text")
        .attr("class", "seconds-label")
        //.attr("x", w/2).attr("y", y + 35)
        .attr("text-anchor", "middle")
        .attr("fill", "black")
        .text("seconds");


    function reltime(t) {return t-data.bounds.min;}
    var last = data.bounds.max - data.bounds.min;
    //last = reltime(d3.max(data.dyhb, function(d){return d.finish_time;}));
    last = last * 1.05;
    // d3.time.scale() has no support for ms or us.
    var xOFF = d3.time.scale().domain([data.bounds.min, data.bounds.max])
                 .range([0,w]);
    var x = d3.scale.linear().domain([-last*0.05, last])
              .range([0,w]);
    function tx(d) { return "translate(" +x(d) + ",0)"; }
    function left(d) { return x(reltime(d.start_time)); }
    function right(d) {
        return d.finish_time ? x(reltime(d.finish_time)) : "1px";
    }
    function width(d) {
        return d.finish_time ? x(reltime(d.finish_time))-x(reltime(d.start_time)) : "1px";
    }
    function halfwidth(d) {
        if (d.finish_time)
            return (x(reltime(d.finish_time))-x(reltime(d.start_time)))/2;
        return "1px";
    }
    function middle(d) {
        if (d.finish_time)
                return (x(reltime(d.start_time))+x(reltime(d.finish_time)))/2;
            else
                return x(reltime(d.start_time)) + 1;
        }
    function color(d) { return data.server_info[d.serverid].color; }
    function servername(d) { return data.server_info[d.serverid].short; }
    function timeformat(duration) {
        // TODO: trim to microseconds, maybe humanize
        return duration;
    }

    function zoomin() {
        //console.log("zoom in!");
        //console.log(x.domain());
        var old = x.domain();
        var quarterwidth = (old[1] - old[0])/4;
        x.domain([old[0]+quarterwidth, old[1]-quarterwidth]);
        //console.log(x.domain());
        redraw();
        //d3.event.preventDefault();
    }

    function zoomout() {
        //console.log("zoom out!");
        var old = x.domain();
        var halfwidth = (old[1] - old[0])/2;
        x.domain([old[0]-halfwidth, old[1]+halfwidth]);
        redraw();
    }

    function pan_and_zoom() {
        //console.log("P", x.domain());
        if (d3.event) d3.event.transform(x);
        //console.log("p", x.domain());
        redraw();
    }

    function redraw() {
        // at this point zoom/pan must be fixed

        var y = 0;
        //chart.select(".dyhb-label")
        //    .attr("x", x(0))//"20px")
        //    .attr("y", y);
        y += 20;

        // DYHB section
        var dyhb_y = d3.scale.ordinal()
                        .domain(d3.range(data.dyhb.length))
                        .rangeBands([y, y+data.dyhb.length*20], .2);
        y += data.dyhb.length*20;
        var dyhb = chart.selectAll("g.dyhb") // one per row
             .data(data.dyhb)
             .attr("transform", function(d,i) {
                       return "translate("+x(reltime(d.start_time))+","
                           +dyhb_y(i)+")";
                   });
        var new_dyhb = dyhb.enter().append("svg:g")
             .attr("class", "dyhb")
             .attr("transform", function(d,i) {
                       return "translate("+x(reltime(d.start_time))+","
                           +dyhb_y(i)+")";
                   })
        ;
        dyhb.select("rect")
             .attr("width", width)
        ;
        new_dyhb.append("svg:rect")
             .attr("width", width)
             .attr("height", dyhb_y.rangeBand())
             .attr("stroke", "black")
             .attr("fill", color)
             .attr("title", function(d){return "shnums: "+d.response_shnums;})
        ;
        new_dyhb.append("svg:text")
             .attr("text-anchor", "end")
             .attr("fill", "#444")
             .attr("x", "-0.3em") // for some reason dx doesn't work
             .attr("dy", "1.0em")
             .attr("font-size", "12px")
             .text(servername)
        ;
        dyhb.select(".rightbox")
             .attr("transform", function(d) {return "translate("+width(d)
                                             +",0)";});
        var dyhb_rightboxes = new_dyhb.append("svg:g")
             .attr("class", "rightbox")
             .attr("transform", function(d) {return "translate("+width(d)
                                             +",0)";})
        ;
        dyhb_rightboxes.append("svg:text")
             .attr("text-anchor", "start")
             .attr("y", dyhb_y.rangeBand())
             .attr("dx", "0.5em")
             .attr("dy", "-0.4em")
             .attr("fill", "#444")
             .attr("font-size", "14px")
             .text(function (d) {return "shnums: "+d.response_shnums;})
        ;

        // read() requests
        chart.select(".read-label")
            .attr("x", "20px")
            .attr("y", y);
        y += 20;
        var read = chart.selectAll("g.read")
             .data(data.read)
             .attr("transform", function(d) {
                       return "translate("+left(d)+","+(y+30*d.row)+")"; });
        read.select("rect")
             .attr("width", width);
        read.select("text")
             .attr("x", halfwidth);
        var new_read = read.enter().append("svg:g")
             .attr("class", "read")
             .attr("transform", function(d) {
                       return "translate("+left(d)+","+(y+30*d.row)+")"; })
        ;
        y += 30*(1+d3.max(data.read, function(d){return d.row;}));
        new_read.append("svg:rect")
             .attr("width", width)
             .attr("height", 20)
             .attr("fill", "red")
             .attr("stroke", "black")
             .attr("title", function(d)
                   {return "read(start="+d.start+", len="+d.length+") -> "
                    +d.bytes_returned+" bytes";})
        ;
        new_read.append("svg:text")
             .attr("x", halfwidth)
             .attr("dy", "0.9em")
             .attr("fill", "black")
             .text(function(d) {return "["+d.start+":+"+d.length+"]";})
        ;

        // segment requests
        chart.select(".segment-label")
            .attr("x", "20px")
            .attr("y", y);
        y += 20;
        var segment = chart.selectAll("g.segment")
             .data(data.segment)
             .attr("transform", function(d) {
                       return "translate("+left(d)+","+(y+30*d.row)+")"; });
        segment.select("rect")
             .attr("width", width);
        segment.select("text")
             .attr("x", halfwidth);
        var new_segment = segment.enter().append("svg:g")
             .attr("class", "segment")
             .attr("transform", function(d) {
                       return "translate("+left(d)+","+(y+30*d.row)+")"; })
        ;
        y += 30*(1+d3.max(data.segment, function(d){return d.row;}));
        new_segment.append("svg:rect")
             .attr("width", width)
             .attr("height", 20)
             .attr("fill", function(d){return d.success ? "#cfc" : "#fcc";})
             .attr("stroke", "black")
             .attr("stroke-width", 1)
             .attr("title", function(d) {
                       return "seg"+d.segment_number+" ["+d.segment_start
                           +":+"+d.segment_length+"] (took "
                           +timeformat(d.finish_time-d.start_time)+")";})
        ;
        new_segment.append("svg:text")
             .attr("x", halfwidth)
             .attr("text-anchor", "middle")
             .attr("dy", "0.9em")
             .attr("fill", "black")
             .text(function(d) {return d.segment_number;})
        ;

        var shnum_colors = d3.scale.category10();

        // block requests
        chart.select(".block-label")
            .attr("x", "20px")
            .attr("y", y);
        y += 20;
        var block_row_to_y = {};
        var dummy = function() {
            var row_y=y;
            for (var group=0; group < data.block_rownums.length; group++) {
                for (var row=0; row < data.block_rownums[group]; row++) {
                    block_row_to_y[group+"-"+row] = row_y;
                    row_y += 12; y += 12;
                }
                row_y += 5; y += 5;
            }
        }();
        var blocks = chart.selectAll("g.block")
             .data(data.block)
             .attr("transform", function(d) {
                       var ry = block_row_to_y[d.row[0]+"-"+d.row[1]];
                       return "translate("+left(d)+"," +ry+")"; });
        blocks.select("rect")
            .attr("width", width);
        blocks.select("text")
            .attr("x", halfwidth);
        var new_blocks = blocks.enter().append("svg:g")
             .attr("class", "block")
             .attr("transform", function(d) {
                       var ry = block_row_to_y[d.row[0]+"-"+d.row[1]];
                       return "translate("+left(d)+"," +ry+")"; })
        ;
        // everything appended to blocks starts at the top-left of the
        // correct per-rect location
        new_blocks.append("svg:rect")
             .attr("width", width)
             .attr("y", function(d) {return (d.response_length > 100) ? 0:3;})
             .attr("height",
                   function(d) {return (d.response_length > 100) ? 10:4;})
             .attr("fill", color)
             .attr("stroke", function(d){return shnum_colors(d.shnum);})
             .attr("stroke-width", 1)
             .attr("title", function(d){
                       return "sh"+d.shnum+"-on-"+d.serverid.slice(0,4)
                           +" ["+d.start+":+"+d.length+"] -> "
                           +d.response_length;})
        ;
        new_blocks.append("svg:text")
             .attr("x", halfwidth)
             .attr("dy", "0.9em")
             .attr("fill", "black")
             .attr("font-size", "8px")
             .attr("text-anchor", "middle")
             .text(function(d) {return "sh"+d.shnum;})
        ;

        var num = d3.format(".4g");

        // horizontal scale markers: vertical lines at rational timestamps
        var rules = chart.selectAll("g.rule")
            .data(x.ticks(10))
            .attr("transform", tx);
        rules.select("text").text(x.tickFormat(10));

        var newrules = rules.enter().insert("svg:g")
              .attr("class", "rule")
              .attr("transform", tx)
        ;

        newrules.append("svg:line")
            .attr("y1", y)
            .attr("y2", y + 6)
            .attr("stroke", "black");
        newrules.append("svg:line")
            .attr("y1", 0)
            .attr("y2", y)
            .attr("stroke", "red")
            .attr("stroke-opacity", .3);
        newrules.append("svg:text")
            .attr("y", y + 9)
            .attr("dy", ".71em")
            .attr("text-anchor", "middle")
            .attr("fill", "black")
            .text(x.tickFormat(10));
        rules.exit().remove();
        chart.select(".seconds-label")
            .attr("x", w/2)
            .attr("y", y + 35);
        y += 45;

        d3.select("#outer_chart").attr("height", y);
        d3.select("#outer_rect").attr("height", y);
        d3.select("#zoom").attr("transform", "translate("+(w-10)+","+10+")");
    }

    var zoom = chart.append("svg:g")
        .attr("id", "zoom");
    zoom.append("svg:rect")
        .attr("width", "20px")
        .attr("height", "20px")
        .attr("stroke", "black")
        .attr("stroke-width", "1")
        .attr("fill", "white")
        .on("click", zoomin)
    ;
    zoom.append("svg:text")
        .attr("x", "10px").attr("y", "0.8em")
        .attr("font-size", "18px")
        .attr("text-anchor", "middle")
        .text("+")
        .on("click", zoomin)
    ;
    zoom.append("svg:rect")
        .attr("y", "30px")
        .attr("width", "20px")
        .attr("height", "20px")
        .attr("stroke", "black")
        .attr("stroke-width", "1")
        .attr("fill", "white")
        .on("click", zoomout)
    ;
    zoom.append("svg:text")
        .attr("y", "30px")
        .attr("x", "10px").attr("dy", "0.8em")
        .attr("font-size", "18px")
        .attr("text-anchor", "middle")
        .text("-")
        .on("click", zoomout)
    ;
    redraw();

    return;
}


function unused() {
    var bounds = { min: data.bounds.min,
                   max: data.bounds.max
                 };
    //bounds.max = data.dyhb[data.dyhb.length-1].finish_time;
    var duration = bounds.max - bounds.min;
    var WIDTH = 600;
    var vis = new pv.Panel().canvas("timeline").margin(30);

    var dyhb_top = 0;
    var read_top = dyhb_top + 30*data.dyhb[data.dyhb.length-1].row+60;
    var segment_top = read_top + 30*data.read[data.read.length-1].row+60;
    var block_top = segment_top + 30*data.segment[data.segment.length-1].row+60;
    var block_row_to_y = {};
    var row_y=0;
    for (var group=0; group < data.block_rownums.length; group++) {
        for (var row=0; row < data.block_rownums[group]; row++) {
            block_row_to_y[group+"-"+row] = row_y;
            row_y += 10;
        }
        row_y += 5;
    }

    var height = block_top + row_y;
    var kx = bounds.min;
    var ky = 1;
    var x = pv.Scale.linear(bounds.min, bounds.max).range(0, WIDTH-40);
    var relx = pv.Scale.linear(0, duration).range(0, WIDTH-40);
    //var y = pv.Scale.linear(-ky,ky).range(0, height);
    //x.nice(); relx.nice();

    /* add the invisible panel now, at the bottom of the stack, so that
    it won't steal mouseover events and prevent tooltips from
    working. */
    var zoomer = vis.add(pv.Panel)
        .events("all")
        .event("mousedown", pv.Behavior.pan())
        .event("mousewheel", pv.Behavior.zoom())
        .event("pan", transform)
        .event("zoom", transform)
    ;

    vis.anchor("top").top(-20).add(pv.Label).text("DYHB Requests");

    /* we use a function for data=relx.ticks() here instead of
     simply .data(relx.ticks()) so that it will be recalculated when
     the scales change (by pan/zoom) */
    var xaxis = vis.add(pv.Rule)
        .data(function() {return relx.ticks();})
        .strokeStyle("#ccc")
        .left(relx)
        .anchor("bottom").add(pv.Label)
        .text(function(d){return relx.tickFormat(d)+"s";});


    vis.height(height);

    function zoomin() {
        var t = zoomer.transform().invert();
        t.k = t.k/1.5;
        zoomer.transform(t.invert());
        zoompan(t);
    }

    function zoomout() {
        var t = zoomer.transform().invert();
        t.k = t.k*1.5;
        zoomer.transform(t.invert());
        zoompan(t);
    }

    function transform() {
        var t = this.transform().invert();
        zoompan(t);
    }
    function zoompan(t) {
        // when t.x=0 and t.k=1.0, left should be bounds.min
        x.domain(bounds.min + (t.x/WIDTH)*duration,
                 bounds.min + t.k*duration + (t.x/WIDTH)*duration);
        relx.domain(0 + t.x/WIDTH*duration,
                    t.k*duration + (t.x/WIDTH)*duration);
        vis.render();
    }

    vis.render();
    $("#zoomin").click(zoomin);
    $("#zoomout").click(zoomout);
}

$(function() {

      $.ajax({url: "event_json",
              method: 'GET',
              dataType: 'json',
              success: onDataReceived });
});

